#!/usr/bin/env python
# coding: utf-8

# In[ ]:


"""
 ------------------------------------------------------------------------
  Python code for computing Fourier-wavelet spectra (FWS)

    RW,RE,PERIOD,COI,SIGNIF,PW,PE = 
          fourierwavelet(DATA,DT,DLON,ZWN,MIN_T,MAX_T,DJ)

    Version 1.0 (released on 14 November 2022)
    Version 1.1 (released on 22 March 2023)
    Provided by Y. Yamazaki [https://orcid.org/0000-0002-7624-4752]
    Available at URL: [https://igit.iap-kborn.de/yamazaki/fourierwavelet]

    Reference: Yamazaki, Y. (2023): Matlab and Python software to 
               compute Fourier-wavelet spectra (fourierwavelet v1.1) 
               using longitude-time data for studying global-scale
               atmospheric waves

 *** Note ***

    Created with Python 3.9.12

    "SciPy" and "NumPy" libraries are required.

    Also, the function "waveletFunctions.py" is required, which is
    provided by Evgeniya Predybaylo based on Torrence and Compo (1998)
    and is available at URL:
    [http://atoc.colorado.edu/research/wavelets/].

    Reference: Torrence, C. and G. P. Compo, 1998: A Practical Guide to
               Wavelet Analysis. Bull. Amer. Meteor. Soc., 79, 61-78.

 *** **** ***
 ------------------------------------------------------------------------
 INPUTS:

    DATA is a space-time data matrix [time x longitude]. Each column
    corresponds to a time series at a fixed longitude. Each row
    corresponds to a longitude distribution at a fixed time. Make sure
    that DATA.shape[0] gives the length in time, and DATA.shape[1] gives
    the length in longitude. The data need to be equally distributed in
    time and longitude. Gaps should be filled by NaN. Also, there should
    be no repetition in longitude. For example, if the first and last
    columns are assigned to the same longitude (e.g., 0 and 360 deg),
    remove the last column, i.e., DATA = DATA[:,0:-1], to avoid the
    repetition.

    DT is sampling time (arbitrary units).

    DLON is longitude spacieng (in units of degrees).

    ZWN is the zonal wavenumber (an integer value, >=0).

 ------------------------------------------------------------------------
 OUTPUTS:

    RW is a matrix [PERIOD x time] of the amplitude of the westward-
    propagating wave component with zonal wavenuber ZWN.

    RE is the same as RW except for the eastward-propagating wave
    component.

    PERIOD is a vector of Fourier periods (in time units).

    COI is a vector for the Cone-of-Influence.
    It gives a vector that contains the maximum period of useful
    information at that particular time.
    Periods greater than COI are subject to edge effects.
    This can be used to plot COI lines on a contour plot, e.g., by doing:

        matplotlib.pylab.contour(time,numpy.log2(period),Rw)
        matplotlib.pylab.plot(time,numpy.log2(coi),'k')

    SIGNIF is a matrix [PERIOD x time] in the same size as RW and RE. 
    RW (or RE) is above the 95% confidence for a red noise background
    (lag-1 autocorrelation = 0.72) where RW (or RE) > SIGNIF.
    This can be used to plot the 95% confidence lines, for example:
    
        matplotlib.pylab.contour(time,numpy.log2(period),Rw/signif,1,'m') 

    PW is a matrix [PERIOD x time] of the phase of the
    westward-propagating wave component with zonal wavenumber ZWN.

    PE is the same as RW except for the eastward-propagating wave
    component.

 ------------------------------------------------------------------------
 OPTIONAL INPUTS:
 
    MIN_T and MAX_T are the minimum and minimum PERIOD to be included, in 
    the same time units as DT. If not specified, the results are output
    for full PERIOD range.

    DJ is the spacing between discrete scales. Default is 0.1.
    A smaller value will give better period resolution, but the
    computation takes more time.

 ------------------------------------------------------------------------
 LICENSE:

    Copyright (C) 2022, Yosuke Yamazaki

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>.

 *** Contact *** 

     Dr. Yosuke Yamazaki
     Leibniz Institute of Atmospheric Physics (IAP) at the University of
     Rostock, Schlossstraße 6, 18225 Kühlungsborn, Germany
     E-mail: yamazaki[AT]iap-kborn[DOT]de

 *** ******* ***

"""
# ----------------------------------------------------------------------------
# import functions
import scipy.io as sio
from scipy import linalg
import numpy as np
from waveletFunctions import wave_signif, wavelet
# ----------------------------------------------------------------------------
def fourierwavelet(data,dt,dlon,zwn,min_t=-1,max_t=np.inf,dj=0.1):
    # ------------------------------------------------------------------------
    if min_t>max_t:
        print('MAX_T must be larger than MIN_T.')
        return
    # ------------------------------------------------------------------------
    # check redundancy in longitude
    if np.array_equal(data[:,0],data[:,-1]):
        print('************************************************************')
        print('The first and last columns of the data are identical.')
        print('If they are assigned to the same longitude (e.g., 0 and')
        print('360 deg), remove the last column, i.e., DATA = DATA[:,0:-1].')
        print('************************************************************')
    # ------------------------------------------------------------------------
    n = data.shape[0] # data length in time
    L = data.shape[1] # data length in longitude
    chk_nan = int(np.isnan(data).any())
    # ------------------------------------------------------------------------
    # Fourier analysis in longitude
    if chk_nan==1:
        # least-squares method (only when NaN exists in data)
        Ck = np.nanmean(data, axis=1)
        Ck = Ck - np.nanmean(Ck)
        Sk = np.zeros(Ck.shape)
        if zwn == 0:
            Ck[np.where(np.isnan(Ck))] = 0
        else:
            lon = np.arange(data.shape[1])*dlon # reconstruction of longitude
            M = np.array([np.cos(2*np.pi*np.absolute(zwn)*lon/360),
                      np.sin(2*np.pi*np.absolute(zwn)*lon/360)])
            M = M.T
            for l in range(n):
                t_data = data[l,:]
                co = np.matmul(np.linalg.pinv(M)[:,np.where(np.isnan(t_data)==0)[0]],
                           t_data[np.where(np.isnan(t_data)==0)])
                Ck[l] = co[0]
                Sk[l] = co[1]
    else:
        # fft method
        Ck = np.mean(data, axis=1)
        Ck = Ck - np.mean(Ck)
        Sk = np.zeros(Ck.shape)
        if zwn != 0:
            for l in range(n):
                f_data = np.fft.fft(data[l,:])/L
                Ck[l] =  2*np.real(f_data[zwn])
                Sk[l] = -2*np.imag(f_data[zwn])
    # ------------------------------------------------------------------------
    # wavelet analysis in time
    param = 6
    [Ck_w0,period,scale,coi] = wavelet(Ck,dt,1,dj,2*dt,
            np.fix((np.log(n*dt/(2*dt))/np.log(2))/dj),'MORLET',param)
    [Sk_w0,_,_,_] = wavelet(Sk,dt,1,dj,2*dt,
            np.fix((np.log(n*dt/(2*dt))/np.log(2))/dj),'MORLET',param)
    Ck_w = Ck_w0*np.dot(np.array([np.sqrt(dt/scale)]).T,np.ones([1,n]))
    Sk_w = Sk_w0*np.dot(np.array([np.sqrt(dt/scale)]).T,np.ones([1,n]))
    # ------------------------------------------------------------------------
    # calculation of amplitude and phase
    Ak =  np.real(Ck_w)
    Bk = -np.imag(Ck_w)
    ak =  np.real(Sk_w)
    bk = -np.imag(Sk_w)
    Rw = .5*np.sqrt((Ak-bk)**2 + ( Bk+ak)**2) # amplitude, westward
    Re = .5*np.sqrt((Ak+bk)**2 + ( Bk-ak)**2) # amplitude, eastward
    Pw = np.arctan2( Bk+ak,Ak-bk)
    Pe = np.arctan2( Bk-ak,Ak+bk)
    n_max = np.where(period > max_t)
    n_max0 = n_max[0]
    n_min = np.where(period < min_t)
    n_min0 = n_min[0]
    if len(n_max0)>0:
        n_max = np.min(n_max0)
    else:
        n_max = len(period)
    if len(n_min0)>0:
        n_min = np.max(n_min0)
    else:
        n_min = 0
    period = period[n_min:n_max+1]
    Rw = Rw[n_min:n_max+1,:]
    Re = Re[n_min:n_max+1,:]
    Pw = Pw[n_min:n_max+1,:]
    Pe = Pe[n_min:n_max+1,:]
    # ------------------------------------------------------------------------
    # 0.05 significance levels
    signif = wave_signif(np.max([np.var(Ck),np.var(Sk)]),dt,
                     scale[n_min:n_max+1],0,0.72,0.95,None,'MORLET')
    signif = .5*np.sqrt(signif*np.sqrt(dt/scale[n_min:n_max+1]))
    signif = np.dot(np.array([signif]).T,np.ones([1,n]))
    # ------------------------------------------------------------------------
    return Rw, Re, period, coi, signif, Pw, Pe