#!/usr/bin/env python
# coding: utf-8

# In[ ]:


"""
 ------------------------------------------------------------------------
  Python example script for computing Fourier-wavelet sepctra using
  geopotential height at 50N and 0.01 hPa during January-May 2016 as
  obtained from the SD/WACCM-X model.
  
  This software is provided by Y. Yamazaki and is available at URL:
  [https://igit.iap-kborn.de/yamazaki/fourierwavelet].
  
  Reference: Yamazaki, Y. (2023): Matlab and Python software to 
             compute Fourier-wavelet spectra (fourierwavelet v1.1) 
             using longitude-time data for studying global-scale
             atmospheric waves
  
  The SD/WACCM-X simulation is that presented by F. Gasperini et al. and
  the data are available at URL: [https://doi.org/10.26024/5b58-nc53].
  
  Reference: Gasperini, F., Liu, H., & McInerney, J. (2020). Preliminary
             evidence of Madden-Julian Oscillation effects on ultrafast
             tropical waves in the thermosphere. Journal of Geophysical
             Research: Space Physics, 125, e2019JA027649.
             https://doi.org/10.1029/2019JA027649.
  
"""
# ----------------------------------------------------------------------------
# import functions

import numpy as np
import matplotlib.pylab as plt
import matplotlib as mpl
from fourierwavelet import fourierwavelet

# ----------------------------------------------------------------------------
# reading input data

GPH = np.loadtxt('SD-WACCM-X_v2.1_GPH.dat');
dt   = 1.0   # time grid resoltuion (day)
dlon = 2.5   # longitude grid resolution (deg)
zwn  = 1     # zonal wavenumber

# ----------------------------------------------------------------------------
# creating vectors for time and longitude

t = np.arange(0,GPH.shape[0],dt) # time
lon = np.arange(0,360,dlon)      # longitude

# ----------------------------------------------------------------------------
# performing Fourier-wavelet analysis

Rw,Re,period,coi,signif,_,_ = fourierwavelet(GPH,dt,dlon,zwn)

# -------------------
# using all optional inputs & outputs
#
# min_t = 3
# max_t = 30
# dj    = 0.05
#
# Rw,Re,period,coi,signif,Pw,Pe = 
#    fourierwavelet(GPH,dt,dlon,zwn,min_t,max_t,dj)

# ----------------------------------------------------------------------------
# plotting

levels = [0,.1,.2,.3,.4,.5,.6,.7]
mpl.rcParams['lines.linewidth'] = 2.5
fig, (ax1, ax2, ax3) = plt.subplots(3,figsize=(10, 10))
fig.tight_layout(pad=5)

plot1 = ax1.pcolor(t,lon,GPH.T,cmap='jet')
ax1.set_ylabel('Longitude (deg.)', fontsize=14)
ax1.set_xlabel('Time (days from 1 Jan 2016)', fontsize=14)
ax1.tick_params(axis='both',labelsize=12)
ax1.set_title('SD/WACCM-X geopotential height (km) at 50N and 0.01 hPa',
              fontweight='bold',fontsize=14)
plt.colorbar(plot1,ax=ax1)

plot2 = ax2.contourf(t,period,Rw,levels)
ax2.set_ylabel('Period (days)', fontsize=14)
ax2.set_xlabel('Time (days from 1 Jan 2016)', fontsize=14)
ax2.tick_params(axis='both',labelsize=12)
ax2.set_title('Fourier-wavelet sepctrum for W1 (km)',
              fontweight='bold',fontsize=14)
ax2.set_ylim([4,28])
ax2.contour(t,period,Rw/signif,[1],colors='white')
ax2.plot(t,coi,'w--')
plt.colorbar(plot2,ax=ax2)

plot3 = ax3.contourf(t,period,Re,levels)
ax3.set_ylabel('Period (days)', fontsize=14)
ax3.set_xlabel('Time (days from 1 Jan 2016)', fontsize=14)
ax3.tick_params(axis='both',labelsize=12)
ax3.set_title('Fourier-wavelet sepctrum for E1 (km)',
              fontweight='bold',fontsize=14)
ax3.set_ylim([4,28])
ax3.contour(t,period,Re/signif,[1],colors='white')
ax3.plot(t,coi,'w--')
plt.colorbar(plot3,ax=ax3)

# In[ ]:
